/*
 * This file is part of MAME4iOS.
 *
 * Copyright (C) 2013 David Valdeita (Seleuco)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 * Linking MAME4iOS statically or dynamically with other modules is
 * making a combined work based on MAME4iOS. Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * In addition, as a special exception, the copyright holders of MAME4iOS
 * give you permission to combine MAME4iOS with free software programs
 * or libraries that are released under the GNU LGPL and with code included
 * in the standard release of MAME under the MAME License (or modified
 * versions of such code, with unchanged license). You may copy and
 * distribute such a system following the terms of the GNU GPL for MAME4iOS
 * and the licenses of the other code concerned, provided that you include
 * the source code of that other code when and as the GNU GPL requires
 * distribution of source code.
 *
 * Note that people who make modified versions of MAME4iOS are not
 * obligated to grant this special exception for their modified versions; it
 * is their choice whether to do so. The GNU General Public License
 * gives permission to release a modified version without this exception;
 * this exception also makes it possible to release a modified version
 * which carries forward this exception.
 *
 * MAME4iOS is dual-licensed: Alternatively, you can license MAME4iOS
 * under a MAME license, as set out in http://mamedev.org/
 */

#import "DefaultOptionController.h"
#import "Globals.h"
#import "Options.h"
#if TARGET_OS_IOS
#import "OptionsController.h"
#elif TARGET_OS_TV
#import "TVOptionsController.h"
#endif
#import "ListOptionController.h"
#import "EmulatorController.h"

#include "myosd.h"

@implementation DefaultOptionController {

    NSArray  *arraySoundValue;
    NSArray  *arrayVideoPriorityValue;
    NSArray  *arrayMainPriorityValue;
    
    NSArray  *arrayMainThreadTypeValue;
    NSArray  *arrayVideoThreadTypeValue;
    
}


- (id)init {
    if (self = [super init]) {

        arraySoundValue = [[NSArray alloc] initWithObjects:@"Off", @"On (11 KHz)", @"On (22 KHz)",@"On (33 KHz)", @"On (44 KHz)", @"On (48 KHz)", nil];
        arrayMainPriorityValue = [[NSArray alloc] initWithObjects:@"0", @"1", @"2", @"3",@"4", @"5", @"6", @"7", @"8", @"9", @"10",nil];
        arrayVideoPriorityValue = [[NSArray alloc] initWithObjects:@"0", @"1", @"2", @"3",@"4", @"5", @"6", @"7", @"8", @"9", @"10",nil];
        
        arrayMainThreadTypeValue = [[NSArray alloc] initWithObjects:@"Normal", @"Real Time RR", @"Real Time FIFO",nil];
        arrayVideoThreadTypeValue = [[NSArray alloc] initWithObjects:@"Normal", @"Real Time RR", @"Real Time FIFO",nil];
        
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"Default Options";
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    
    return 3;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    switch (section)
    {
        case 0: return 3;
        case 1: return 4;
        case 2: return 7;
    }
    return -1;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    
    switch (section)
    {
        case 0: return @"Vector Defaults";
        case 1: return @"Game Defaults";
        case 2: return @"App Defaults";
    }
    return @"Error!";
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    UITableViewCell* cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:nil];

    cell.accessoryType = UITableViewCellAccessoryNone;
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.accessoryView = nil;
    
    Options *op = [[Options alloc] init];
    
    switch (indexPath.section)
    {
        case 0:
        {
            switch (indexPath.row)
            {   case 0:
                {
                    cell.textLabel.text = @"Beam 2x";
                    cell.accessoryView = [self optionSwitchForKey:@"vbean2x"];
                    break;
                }
                    
                case 1:
                {
                    cell.textLabel.text = @"Antialias";
                    cell.accessoryView = [self optionSwitchForKey:@"vantialias"];
                    break;
                }
                case 2:
                {
                    cell.textLabel.text = @"Flicker";
                    cell.accessoryView = [self optionSwitchForKey:@"vflicker"];
                    break;
                }
            }
            break;
        }
        case 1:
        {
            switch (indexPath.row)
            {   case 0:
                {
                    cell.textLabel.text   = @"Sound";
                    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
                    cell.detailTextLabel.text = [arraySoundValue optionAtIndex:op.soundValue];
                    break;
                }
                    
                case 1:
                {
                    cell.textLabel.text = @"Cheats";
                    cell.accessoryView = [self optionSwitchForKey:@"cheats"];
                    break;
                }
                case 2:
                {
                    cell.textLabel.text   = @"Force 60Hz Sync";
                    cell.accessoryView = [self optionSwitchForKey:@"vsync"];
                    break;
                }
                case 3:
                {
                    cell.textLabel.text   = @"Save Hiscores";
                    cell.accessoryView = [self optionSwitchForKey:@"hiscore"];
                    break;
                }
            }
            break;
        }
        case 2:
        {
            switch (indexPath.row)
            {   case 0:
                {
                    cell.textLabel.text   = @"Native TV-OUT";
                    cell.accessoryView = [self optionSwitchForKey:@"tvoutNative"];
                    break;
                }
                    
                case 1:
                {
                    cell.textLabel.text   = @"Threaded Video";
                    cell.accessoryView = [self optionSwitchForKey:@"threaded"];
                    break;
                }
                case 2:
                {
                    cell.textLabel.text   = @"Video Thread Priority";
                    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
                    cell.detailTextLabel.text = [arrayVideoPriorityValue optionAtIndex:op.videoPriority];
                    break;
                }
                case 3:
                {
                    cell.textLabel.text   = @"Video Thread Type";
                    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
                    cell.detailTextLabel.text = [arrayVideoThreadTypeValue optionAtIndex:op.videoThreadType];
                    break;
                }
                case 4:
                {
                    cell.textLabel.text   = @"Double Buffer";
                    cell.accessoryView = [self optionSwitchForKey:@"dblbuff"];
                    break;
                }
                case 5:
                {
                    cell.textLabel.text   = @"Main Thread Priority";
                    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
                    cell.detailTextLabel.text = [arrayMainPriorityValue optionAtIndex:op.mainPriority];
                    break;
                }
                case 6:
                {
                    cell.textLabel.text   = @"Main Thread Type";
                    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
                    cell.detailTextLabel.text = [arrayMainThreadTypeValue optionAtIndex:op.mainThreadType];
                    break;
                }
            }
            break;
        }
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSUInteger row = [indexPath row];
    NSUInteger section = [indexPath section];
    
#if TARGET_OS_TV
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    [self toggleOptionSwitch:cell.accessoryView];
#endif
    
    if(section==1 && row==0)
    {
        ListOptionController *listController = [[ListOptionController alloc] initWithStyle:UITableViewStyleGrouped
                                                                                      type:kTypeSoundValue list:arraySoundValue];
        [[self navigationController] pushViewController:listController animated:YES];
    }
    if (section==2 && row==2){
        ListOptionController *listController = [[ListOptionController alloc] initWithStyle:UITableViewStyleGrouped
                                                                                      type:kTypeVideoPriorityValue list:arrayVideoPriorityValue];
        [[self navigationController] pushViewController:listController animated:YES];
    }
    if (section==2 && row==3){
        ListOptionController *listController = [[ListOptionController alloc] initWithStyle:UITableViewStyleGrouped
                                                                                      type:kTypeVideoThreadTypeValue list:arrayVideoThreadTypeValue];
        [[self navigationController] pushViewController:listController animated:YES];
    }
    if (section==2 && row==5){
        ListOptionController *listController = [[ListOptionController alloc] initWithStyle:UITableViewStyleGrouped
                                                                                      type:kTypeMainPriorityValue list:arrayMainPriorityValue];
        [[self navigationController] pushViewController:listController animated:YES];
    }
    if (section==2 && row==6){
        ListOptionController *listController = [[ListOptionController alloc] initWithStyle:UITableViewStyleGrouped
                                                                                      type:kTypeMainThreadTypeValue list:arrayMainThreadTypeValue];
        [[self navigationController] pushViewController:listController animated:YES];
    }

}


@end
